/*
 * OHCI HCD (Host Controller Driver) for USB.
 *
 * (C) Copyright 1999 Roman Weissgaerber <weissg@vienna.at>
 * (C) Copyright 2000-2002 David Brownell <dbrownell@users.sourceforge.net>
 * (C) Copyright 2002 Hewlett-Packard Company
 *
 * Bus Glue for INCAIP2
 *
 * Written by Christopher Hoover <ch@hpl.hp.com>
 * Based on fragments of previous driver by Rusell King et al.
 *
 * Modified for LH7A404 from ohci-sa1111.c
 *  by Durgesh Pattamatta <pattamattad@sharpsec.com>
 * Modified for INCAIP2
 * This file is licenced under the GPL.
 */

#include <linux/platform_device.h>
#include <linux/signal.h>

#if defined(CONFIG_INCAIP2)       /* Some platform related parameter */
#include <asm/incaip2/incaip2.h>
#include <asm/incaip2/irq.h>
#define IFX_INCAIP2_USB_REG__BASE 0x14103400
#define IFX_INCAIP2_USB_REG__LEN 0xC00
#define IFX_INCAIP2_USB_IRQ INT_NUM_IM4_IRL21 
#endif

#define IFX_USB_REG_BASE IFX_INCAIP2_USB_REG__BASE
#define IFX_USB_REG_LEN IFX_INCAIP2_USB_REG__LEN
#define IFX_USB_IRQ IFX_INCAIP2_USB_IRQ

#define INCA_IP2_PMS_PMS_GEN_USB   (1 << 17)

extern int usb_disabled(void);

/* OHCI (USB full speed host controller) */
static struct resource incaip2_usb_ohci_resources[] = {
	[0] = {
		.start		= IFX_USB_REG_BASE,
		.end		= IFX_USB_REG_BASE + IFX_USB_REG_LEN -1,
		.flags		= IORESOURCE_MEM,
	},
	[1] = {
		.start		= IFX_USB_IRQ,
		.end		= IFX_USB_IRQ,
		.flags		= IORESOURCE_IRQ,
	},
};

static u64 ohci_dmamask = 0xffffffffUL;

static struct platform_device incaip2_usb_ohci_device = {
	.name		= "incaip2-ohci",
	.id		= -1,
	.dev = {
//		.dma_mask		= NULL,
//		.coherent_dma_mask	= 0,
		.dma_mask		= &ohci_dmamask,
		.coherent_dma_mask	= 0xffffffff,
	},
	.num_resources	= ARRAY_SIZE(incaip2_usb_ohci_resources),
	.resource	= incaip2_usb_ohci_resources,
};

static struct platform_device *incaip2_platform_devices[] = {
         &incaip2_usb_ohci_device};

int incaip2_platform_init(void)
{
   printk("In platform Init\n");
	return platform_add_devices(incaip2_platform_devices, ARRAY_SIZE(incaip2_platform_devices));

}

/*-------------------------------------------------------------------------*/

static void incaip2_start_ohc(struct platform_device *dev)
{
	printk(KERN_DEBUG __FILE__
		": starting INCAIP2 OHCI USB Controller\n");
	
   /* enable host controller */
	switch ( INCA_IP2_MPS_CHIPID_VERSION_GET(*INCA_IP2_MPS_CHIPID) ) {
	case 1: /* V1.1 */
	case 2:	/* V1.2 */
		err("USB runs on INCA-IP2 v1.3 and higher!");
		/*TODO return ;*/
	default:
		/* Enable HW module */
		*INCA_IP2_PMS_PMS_GEN |= INCA_IP2_PMS_PMS_GEN_USB;
     // *INCA_IP2_MPS_CHIPID &= ~0x800;/*usb controller enabled*/
     //  *INCA_IP2_CGU_XBARCR  = 0x9;
      printk("the Value of DTO1 reg is %x\n",*INCA_IP2_CGU_DTO1CR);   
      printk("the Value of XBAR reg is %x\n",*INCA_IP2_CGU_XBARCR);   
      printk("the Value of PMS Status reg is %x\n",*INCA_IP2_PMS_PMS_SR);   
      printk("the Value of PMS GEN reg is %x\n",*INCA_IP2_PMS_PMS_GEN);   
      printk("the Value of PMS CFG reg is %x\n",*INCA_IP2_PMS_PMS_CFG);   
	}
	printk(KERN_DEBUG __FILE__
	": Clock to USB host has been enabled \n");
}

static void incaip2_stop_ohc(struct platform_device *dev)
{
	printk(KERN_DEBUG __FILE__
	       ": stopping INCAIP2 OHCI USB Controller\n");
}


/*-------------------------------------------------------------------------*/

/* configure so an HC device and id are always provided */
/* always called with process context; sleeping is OK */


/**
 * usb_ohci_incaip2_probe - initialize INCAIP2-based HCDs
 * Context: !in_interrupt()
 *
 * Allocates basic resources for this USB host controller, and
 * then invokes the start() method for the HCD associated with it
 * through the hotplug entry's driver_data.
 *
 */
static int usb_ohci_incaip2_probe(const struct hc_driver *driver,
			  struct platform_device *dev)
{
	int retval;
	struct usb_hcd *hcd;
   printk("In usb_ohci_incaip2_probe \n");
	if (dev->resource[1].flags != IORESOURCE_IRQ) {
		printk("resource[1] is not IORESOURCE_IRQ\n");
		return -ENOMEM;
	}

	hcd = usb_create_hcd(driver, &dev->dev, "incaip2");
	if (!hcd)
		return -ENOMEM;
	hcd->rsrc_start = dev->resource[0].start;
	hcd->rsrc_len = dev->resource[0].end - dev->resource[0].start + 1;

	if (!request_mem_region(hcd->rsrc_start, hcd->rsrc_len,hcd_name)) {
		printk("request_mem_region failed\n");
		retval = -EBUSY;
		goto err1;
	}

	hcd->regs = ioremap_nocache(hcd->rsrc_start, hcd->rsrc_len);
	if (!hcd->regs) {
		printk("ioremap failed\n");
		retval = -ENOMEM;
		goto err2;
	}
   printk("The Mapped address is %x\n",(u32) hcd->regs);
	incaip2_start_ohc(dev);
	ohci_hcd_init(hcd_to_ohci(hcd));

//	retval = usb_add_hcd(hcd, dev->resource[1].start, IRQF_DISABLED | IRQF_SHARED);
	retval = usb_add_hcd(hcd, dev->resource[1].start, IRQF_DISABLED);
	if (retval == 0){
      printk("Out Success usb_ohci_incaip2_probe \n");
		return retval;
   }
   printk("Out Failed usb_ohci_incaip2_probe \n");
	incaip2_stop_ohc(dev);
	iounmap(hcd->regs);
 err2:
	release_mem_region(hcd->rsrc_start, hcd->rsrc_len);
 err1:
	usb_put_hcd(hcd);
	return retval;
}


/* may be called without controller electrically present */
/* may be called with controller, bus, and devices active */

/**
 * usb_hcd_INCAIP2_remove - shutdown processing for incaip2-based HCDs
 * @dev: USB Host Controller being removed
 * Context: !in_interrupt()
 *
 * Reverses the effect of usb_hcd_incaip2_probe(), first invoking
 * the HCD's stop() method.  It is always called from a thread
 * context, normally "rmmod", "apmd", or something similar.
 *
 */
static void usb_ohci_incaip2_remove(struct usb_hcd *hcd, struct platform_device *dev)
{
   printk("In usb_ohci_incaip2_remove\n");
	usb_remove_hcd(hcd);
	incaip2_stop_ohc(dev);
	iounmap(hcd->regs);
	release_mem_region(hcd->rsrc_start, hcd->rsrc_len);
	usb_put_hcd(hcd);
}

/*-------------------------------------------------------------------------*/

static int __devinit
ohci_incaip2_start (struct usb_hcd *hcd)
{
	struct ohci_hcd	*ohci = hcd_to_ohci (hcd);
	int		ret;

	printk("In ohci_incaip2_start \n");

	if ((ret = ohci_init (ohci)) < 0)
		return ret;


	if ((ret = ohci_run (ohci)) < 0) {
		printk("can't start %s", hcd->self.bus_name);
		ohci_stop (hcd);
		return ret;
	}

	return 0;
}

/*------------------------------------------------------------------------------------------*/
static const struct hc_driver ohci_incaip2_hc_driver = {
	.description =		hcd_name,
	.product_desc =		"IncaIP-2 OHCI",
	.hcd_priv_size =	sizeof(struct ohci_hcd),

	/*
	 * generic hardware linkage
	 */
	.irq =			ohci_irq,
	.flags =		   HCD_USB11 | HCD_MEMORY,
//	.flags =		   HCD_USB11,
//	.flags =		   HCD_USB2,

	/*
	 * basic lifecycle operations
	 */
	.start =		ohci_incaip2_start,
	.stop =			ohci_stop,
	.shutdown =		ohci_shutdown,

	/*
	 * managing i/o requests and associated device resources
	 */
	.urb_enqueue =		ohci_urb_enqueue,
	.urb_dequeue =		ohci_urb_dequeue,
	.endpoint_disable =	ohci_endpoint_disable,

	/*
	 * scheduling support
	 */
	.get_frame_number =	ohci_get_frame,

	/*
	 * root hub support
	 */
	.hub_status_data =	ohci_hub_status_data,
	.hub_control =		ohci_hub_control,
	.hub_irq_enable =	ohci_rhsc_enable,
#ifdef	CONFIG_PM
	.bus_suspend =		ohci_bus_suspend,
	.bus_resume =		ohci_bus_resume,
#endif
	.start_port_reset =	ohci_start_port_reset,
};





/*-------------------------------------------------------------------------*/

static int ohci_hcd_incaip2_drv_probe(struct platform_device *pdev)
{
	int ret;
   printk(" in ohci_hcd_incaip2_drv_probe\n");
   if (usb_disabled()){
     printk("USB is Disabled\n");
     return -ENODEV;
   }
   //device_init_wakeup(&pdev->dev, 1);
	ret = usb_ohci_incaip2_probe(&ohci_incaip2_hc_driver, pdev);
	return ret;
}

static int ohci_hcd_incaip2_drv_remove(struct platform_device *pdev)
{
	struct usb_hcd *hcd = platform_get_drvdata(pdev);
   printk(" in ohci_hcd_incaip2_drv_remove\n");
	usb_ohci_incaip2_remove(hcd, pdev);
	return 0;
}

/*-----------------------------------------------------------------------*/
static struct platform_driver ohci_hcd_incaip2_driver = {
	.probe		= ohci_hcd_incaip2_drv_probe,
	.remove		= ohci_hcd_incaip2_drv_remove,
	.shutdown	= usb_hcd_platform_shutdown,
	.driver		= {
		.name	= "incaip2-ohci",
		.owner	= THIS_MODULE,
	},
};

static int __init ohci_hcd_incaip2_init (void)
{
     int ret;

     ret = platform_driver_register(&ohci_hcd_incaip2_driver);
     if (ret == 0) {
          ret = platform_add_devices(incaip2_platform_devices, ARRAY_SIZE(incaip2_platform_devices));
     }
     return ret;
}

static void __exit ohci_hcd_incaip2_cleanup (void)
{
	platform_driver_unregister(&ohci_hcd_incaip2_driver);
}

module_init (ohci_hcd_incaip2_init);
module_exit (ohci_hcd_incaip2_cleanup);

