

/* include/video/epson13733.h
 *
 * (c) 2008 Siemens Enterprise Communications
 * (c) 2008 Karl Bittnar <karl.bittnar@siemens.com>
 * (c) 2008 Viktor Karell <viktor.karell@siemens.com>
 *
 * Header file for Epson S1D13733 driver code
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License. See the file COPYING in the main directory of this archive for
 * more details.
 */

#ifndef	EPSON13733_H
#define	EPSON13733_H

#define S1D_FBID			"EPSON13733"
#define S1D_DEVICENAME			"epson13733fb"
#define S1D_PRODUCT_INFO		(0xD495 >> 2)	/* expected buffer size & product code for s1d13733 */

#define S1D_PHYSICAL_REG_ADDR		0x11000000L
#define S1D_PHYSICAL_REG_SIZE		200L
#define S1D_PHYSICAL_VMEM_ADDR		0x11100000L
#define S1D_PHYSICAL_VMEM_SIZE		868352L		/* display buffer size = 0xD4 * 4 K = 848 K */

#define S1D_DISPLAY_BPP			16

#define S1D_DISPLAY_WIDTH_WP3		320
#define S1D_DISPLAY_HEIGHT_WP3		240
#define S1D_DISPLAY_SCANLINE_BYTES_WP3	640

#define S1D_DISPLAY_WIDTH_WP4		640
#define S1D_DISPLAY_HEIGHT_WP4		480
#define S1D_DISPLAY_SCANLINE_BYTES_WP4	1280

#define S1D_DISPLAY_PCLK                15007744L

/* register definitions */

#define S1DREG_PRODUCT_INFO		0x0000	/* buffer size & product info Register */
#define S1DREG_PS_CNF			0x01F0	/* Power Save Configuration Register */
#define S1DREG_COM_DISP_MODE		0x01FC	/* Common Display Mode Register */

#define SIEMENS_FPGA		(KSEG1+0x1c000000)
#define FPGA_DISP_RESET		((volatile u8*)((SIEMENS_FPGA) + 0x0014))
#define FPGA_PAN_CNTRL		((volatile u8*)((SIEMENS_FPGA) + 0x001E))

#define FPGA_PANEL_POWER_EN_BIT		(1<<0)
#define FPGA_PANEL_SPI_EN_BIT		(1<<1)
#define FPGA_PANEL_SPI_CLK_BIT		(1<<2)
#define FPGA_PANEL_SPI_DATA_BIT		(1<<3)
#define FPGA_PANEL_SPI_DIR_BIT		(1<<4)
#define FPGA_PANEL_DEFAULT_WRITE_BITS	(FPGA_PANEL_POWER_EN_BIT | FPGA_PANEL_SPI_EN_BIT | FPGA_PANEL_SPI_DIR_BIT)
#define FPGA_PANEL_DEFAULT_READ_BITS	(FPGA_PANEL_POWER_EN_BIT | FPGA_PANEL_SPI_EN_BIT)


struct epson13733fb_par {
	void __iomem	*regs;
	u32		pseudo_palette[16];
};

struct epson13733fb_regval {
	u16	reg;
	u16	value;
};

struct epson13733fb_initreg {
	const struct epson13733fb_regval	*initregs_data;
	const unsigned int			initregs_size;
};

struct epson13733fb_splash_data {
	unsigned int	width;
	unsigned int	height;
	unsigned int	pixel_data[];
};

#define S1D_REGRESERVED                 0xFFF0
#define S1D_REGDELAYPLL                 0xFFFC
#define S1D_REGDELAYOFF                 0xFFFD
#define S1D_REGDELAYON                  0xFFFE

typedef unsigned short S1D_INDEX;
typedef unsigned short S1D_VALUE;
typedef struct
{
    S1D_INDEX Index;
    S1D_VALUE Value;
} S1D_REGS;

#define S1D_INSTANTIATE_REGISTERS(scope_prefix,variable_name)  \
  scope_prefix S1D_REGS variable_name[] =  \
  {  \
    { 0x0006,           0x0003 },   /* Bus Timeout Reset Register                         */  \
    { 0x0014,           0x0411 },   /* Miscellaneous Configuration Register               */  \
    { 0x0012,           0x0001 },   /* PLL Setting Register 3                             */  \
    { 0x000C,           0x1027 },   /* PLL Setting Register 0                             */  \
    { 0x000E,           0x7390 },   /* PLL Setting Register 1                             */  \
    { 0x0010,           0x0103 },   /* PLL Setting Register 2                             */  \
    { 0x0018,           0x0000 },   /* System Clock Setting Register                      */  \
    { 0x0012,           0x0000 },   /* PLL Setting Register 3                             */  \
    { 0x001A,           0x0000 },   /* PLL2 Setting Register 0 [RESERVED]                 */  \
    { 0x001C,           0x0000 },   /* PLL2 Setting Register 1 [RESERVED]                 */  \
    { 0x001E,           0x0000 },   /* PLL2 Setting Register 2 [RESERVED]                 */  \
    { S1D_REGDELAYPLL,  0x0064 },   /* PLL REGISTER PROGRAMMING DELAY (in ms)             */  \
    { 0x0014,           0x0410 },   /* Miscellaneous Configuration Register               */  \
    { 0x0300,           0x0000 },   /* GPIO Configuration Register 0                      */  \
    { 0x0302,           0x0000 },   /* GPIO Configuration Register 1                      */  \
    { 0x0304,           0x0000 },   /* GPIO Input Control Register 0                      */  \
    { 0x0306,           0x0000 },   /* GPIO Input Control Register 1                      */  \
    { 0x0308,           0xFFFF },   /* GPIO Pull Down Control Register 0                  */  \
    { 0x030A,           0x003F },   /* GPIO Pull Down Control Register 1                  */  \
    { 0x030C,           0x0000 },   /* GPIO Status Register 0                             */  \
    { 0x030E,           0x0000 },   /* GPIO Status Register 1                             */  \
    { 0x0022,           0x0000 },   /* Indirect Interface Memory Address Register 0       */  \
    { 0x0024,           0x0000 },   /* Indirect Interface Memory Address Register 2       */  \
    { 0x0030,           0x0501 },   /* LCD Interface Clock Setting Register               */  \
    { 0x0032,           0x0031 },   /* LCD Interface Panel Setting Register               */  \
    { 0x0034,           0x0000 },   /* LCD Interface Command Port Register                */  \
    { 0x0036,           0x0000 },   /* LCD Interface Parameter Port Register              */  \
    { 0x003A,           0x0000 },   /* LCD Interface Frame Transfer Register              */  \
    { 0x003C,           0x0000 },   /* LCD Interface Transfer Setting Register            */  \
    { 0x0040,           0x0063 },   /* LCD1 Horizontal Total Register                     */  \
    { 0x0042,           0x013F },   /* LCD1 Horizontal Display Period Register            */  \
    { 0x0044,           0x0032 },   /* LCD1 Horizontal Display Period Start Position Register */  \
    { 0x0046,           0x00DF },   /* LCD1 FPLINE Register                               */  \
    { 0x0048,           0x003B },   /* LCD1 FPLINE Pulse Position Register                */  \
    { 0x004A,           0x020C },   /* LCD1 Vertical Total Register                       */  \
    { 0x004C,           0x01DF },   /* LCD1 Vertical Display Period Register              */  \
    { 0x004E,           0x002C },   /* LCD1 Vertical Display Period Start Position Register */  \
    { 0x0050,           0x0087 },   /* LCD1 FPFRAME Register                              */  \
    { 0x0052,           0x0002 },   /* LCD1 FPFRAME Pulse Position Register               */  \
    { 0x0054,           0x0003 },   /* LCD1 Serial Interface Setting Register             */  \
    { 0x0056,           0x0006 },   /* LCD1 Parallel Interface Setting Register           */  \
    { 0x0058,           0x0000 },   /* LCD2 Horizontal Display Period Register            */  \
    { 0x005A,           0x0000 },   /* LCD2 Vertical Display Period Register              */  \
    { 0x005C,           0x0001 },   /* LCD2 Serial Interface Setting Register             */  \
    { 0x005E,           0x0000 },   /* LCD2 Parallel Interface Setting Register           */  \
    { 0x0180,           0x0FC0 },   /* I2C Status Register                                */  \
    { 0x0200,           0x0035 },   /* Display Mode Setting Register 0                    */  \
    { 0x0202,           0x0400 },   /* Display Mode Setting Register 1                    */  \
    { 0x0204,           0x0000 },   /* Transparent Overlay Key Color Red Data Register    */  \
    { 0x0206,           0x0000 },   /* Transparent Overlay Key Color Green Data Register  */  \
    { 0x0208,           0x0000 },   /* Transparent Overlay Key Color Blue Data Register   */  \
    { 0x0210,           0x0000 },   /* Main Window Display Start Address Register 0       */  \
    { 0x0212,           0x0000 },   /* Main Window Display Start Address Register 1       */  \
    { 0x0216,           0x0500 },   /* Main Window Line Address Offset Register           */  \
    { 0x0218,           0xC000 },   /* PIP+ Display Start Address Register 0              */  \
    { 0x021A,           0x0004 },   /* PIP+ Display Start Address Register 1              */  \
    { 0x021E,           0x0280 },   /* PIP+ Window Line Address Offset Register           */  \
    { 0x0220,           0x0000 },   /* PIP+ X Start Positions Register                    */  \
    { 0x0222,           0x0000 },   /* PIP+ Y Start Positions Register                    */  \
    { 0x0224,           0x013F },   /* PIP+ X End Positions Register                      */  \
    { 0x0226,           0x00EF },   /* PIP+ Y End Positions Register                      */  \
    { 0x022A,           0x0000 },   /* Back Buffer Display Start Address Register 0       */  \
    { 0x022C,           0x0000 },   /* Back Buffer Display Start Address Register 1       */  \
    { 0x0240,           0x0545 },   /* YUV/RGB Converter 1 Translate Mode Register        */  \
    { 0x0242,           0xC000 },   /* YUV/RGB Converter 1 Write Start Address Register 0 */  \
    { 0x0244,           0x0004 },   /* YUV/RGB Converter 1 Write Start Address Register 1 */  \
    { 0x0246,           0x0000 },   /* YUV/RGB Converter 1 Write Start Address 1 Register 0 */  \
    { 0x0248,           0x0000 },   /* YUV/RGB Converter 1 Write Start Address 1 Register 1 */  \
    { 0x024A,           0x0000 },   /* YUV/RGB Converter 1 UV Data Clip Register          */  \
    { 0x024C,           0x00A0 },   /* YUV/RGB Converter 1 Rectangular Pixel Width Register */  \
    { 0x024E,           0x00A0 },   /* YUV/RGB Converter 1 Rectangular Line Address Offset Register */  \
    { 0x0260,           0x0005 },   /* RGB/YUV Converter Configuration Register           */  \
    { 0x0264,           0x0000 },   /* RGB/YUV Converter Horizontal Display Size Register */  \
    { 0x0266,           0x0000 },   /* RGB/YUV Converter Vertical Display Size Register   */  \
    { 0x0284,           0x0000 },   /* Main Window Swivel View Offset Register 0          */  \
    { 0x0286,           0x0000 },   /* Main Window Swivel View Offset Register 1          */  \
    { 0x028C,           0x0000 },   /* Main Window Swivel View Offset Register 0          */  \
    { 0x028E,           0x0000 },   /* PIP Window Swivel View Offset Register 1           */  \
    { 0x0310,           0x0000 },   /* Average Overlay Key Color Red Data Register        */  \
    { 0x0312,           0x0000 },   /* Average Overlay Key Color Green Data Register      */  \
    { 0x0314,           0x0000 },   /* Average Overlay Key Color Blue Data Register       */  \
    { 0x0316,           0x0000 },   /* AND Overlay Key Color Red Data Register            */  \
    { 0x0318,           0x0000 },   /* AND Overlay Key Color Green Data Register          */  \
    { 0x031A,           0x0000 },   /* AND Overlay Key Color Blue Data Register           */  \
    { 0x031C,           0x0000 },   /* OR Overlay Key Color Red Data Register             */  \
    { 0x031E,           0x0000 },   /* OR Overlay Key Color Green Data Register           */  \
    { 0x0320,           0x0000 },   /* OR Overlay Key Color Blue Data Register            */  \
    { 0x0322,           0x0000 },   /* INV Overlay Key Color Red Data Register            */  \
    { 0x0324,           0x0000 },   /* INV Overlay Key Color Green Data Register          */  \
    { 0x0326,           0x0000 },   /* INV Overlay Key Color Blue Data Register           */  \
    { 0x0328,           0x0000 },   /* Overlay Miscellaneous Register                     */  \
    { 0x092C,           0xFFFC },   /* Resizer Camera Input Line Buffer SRAM End Address Register 0 */  \
    { 0x092E,           0x000C },   /* Resizer Camera Input Line Buffer SRAM End Address Register 1 */  \
    { 0x0930,           0x0001 },   /* Global Resize Control Register                     */  \
    { 0x0932,           0x0030 },   /* Resizer Mode Select Register                       */  \
    { 0x0940,           0x0000 },   /* Preview Trimming Start X Position Register         */  \
    { 0x0942,           0x0000 },   /* Preview Trimming Start Y Position Register         */  \
    { 0x0944,           0x027F },   /* Preview Trimming End X Position Register           */  \
    { 0x0946,           0x01DF },   /* Preview Trimming End Y Position Register           */  \
    { 0x0948,           0x0000 },   /* Capture Trimming Start X Position Register         */  \
    { 0x094A,           0x0000 },   /* Capture Trimming Start Y Position Register         */  \
    { 0x094C,           0x027F },   /* Capture Trimming End X Position Register           */  \
    { 0x094E,           0x01DF },   /* Capture Trimming End Y Position Register           */  \
    { 0x0950,           0x0001 },   /* View Resizer Control Register                      */  \
    { 0x0954,           0x0140 },   /* View Resizer Horizontal Output Size Register       */  \
    { 0x0956,           0x00F0 },   /* View Resizer Vertical Output Size Register         */  \
    { 0x095C,           0x2000 },   /* View Resizer Line Buffer SRAM Start Address Register */  \
    { 0x0960,           0x0000 },   /* Capture Resizer Control Register                   */  \
    { 0x0964,           0x0280 },   /* Capture Resizer Horizontal Output Size Register    */  \
    { 0x0966,           0x01E0 },   /* Capture Resizer Vertical Output Size Register      */  \
    { 0x096C,           0x0000 },   /* Capture Resizer Line Buffer SRAM Start Address Register */  \
    { 0x0974,           0x0000 },   /* SRAM Read Width Register                           */  \
    { 0x0976,           0x0000 },   /* SRAM Read Height Register                          */  \
    { 0x0978,           0x0000 },   /* SRAM Read Start Address Register 0                 */  \
    { 0x097A,           0x0000 },   /* SRAM Read Start Address Register 1                 */  \
    { 0x097C,           0x0000 },   /* SRAM Read Line Offset Register                     */  \
    { 0x0980,           0x0000 },   /* JPEG Control Register                              */  \
    { 0x09A0,           0x0020 },   /* JPEG FIFO Control Register                         */  \
    { 0x09A4,           0x001F },   /* JPEG FIFO Size Register                            */  \
    { 0x09B0,           0x0000 },   /* Encode Size Limit Register 0                       */  \
    { 0x09B2,           0x0020 },   /* Encode Size Limit Register 1                       */  \
    { 0x09BC,           0x0140 },   /* JPEG FIFO Start Address Register 0 [READONLY]      */  \
    { 0x09BE,           0x0000 },   /* JPEG FIFO Start Address Register 1 [READONLY]      */  \
    { 0x09D0,           0x0000 },   /* JPEG Line Buffer Configuration Register            */  \
    { 0x09D2,           0x0000 },   /* JPEG Line Buffer Start Address Register            */  \
    { 0x0100,           0x0002 },   /* Camera Clock Setting Register                      */  \
    { 0x0110,           0x1800 },   /* Camera Mode Setting Register                       */  \
    { 0x0102,           0x0012 },   /* Camera Signal Setting Register                     */  \
    { 0x0112,           0x0000 },   /* Camera Frame Setting Register                      */  \
    { 0x0114,           0x0000 },   /* Camera Control Register [WRITEONLY]                */  \
    { 0x0A02,           0x0000 },   /* Interrupt Control Register                         */  \
    { 0x0A06,           0x0000 },   /* LCD Interrupt Flag Register                        */  \
    { 0x0A08,           0x0000 },   /* LCD Interrupt Control Register                     */  \
    { 0x0A0A,           0x0000 },   /* Host Interface Interrupt Flag Register             */  \
    { 0x0A0C,           0x0000 },   /* Host Interface Interrupt Control Register          */  \
    { 0x0A0E,           0x1F1F },   /* Cycle Cycle Time Out Control Register              */  \
    { 0x6100,           0x0021 },   /* SD Memory Card Control Register 0                  */  \
    { S1D_REGDELAYON,   0x0000 }    /* LCD PANEL POWER ON DELAY (in ms)                   */  \
  }

struct epson13733fb_pdata {
	int					WP_type;
	const struct epson13733fb_initreg	*initregs_array;
	const unsigned int			initregs_array_size;
	const struct epson13733fb_splash_data	*splash_data;
	void					(*platform_init_video)(void);
	unsigned long				window_start_offset;
};

#endif
